package egg_point_coefficient

import (
	"applet/app/db"
	"applet/app/e"
	md "applet/app/md/egg_point_coefficient"
	"applet/app/svc"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"github.com/gin-gonic/gin"
	"time"
)

// EggPointCoefficientBatchAdd
// @Summary      制度中心-蛋蛋分区间系数管理-区间系数(新增)
// @Tags         蛋蛋分区间系数管理
// @Description  区间系数(新增 支持批量新增)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req    body	md.EggPointCoefficientBatchAddReq 	true  "需要新增的内容"
// @Success      200   {int}   	"新增数据条数"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggPointCoefficient/add [POST]
func EggPointCoefficientBatchAdd(c *gin.Context) {
	var req md.EggPointCoefficientBatchAddReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	var affected int64
	if len(req.List) == 0 {
		e.OutSuc(c, affected, nil)
		return
	}
	now := time.Now()
	managements := make([]model.EggPointPartitionCoefficientManagement, len(req.List))
	for i, node := range req.List {
		managements[i] = model.EggPointPartitionCoefficientManagement{
			StartScore:  node.StartScore,
			EndScore:    node.EndScore,
			Coefficient: node.Coefficient,
			CreateAt:    now.Format("2006-01-02 15:04:05"),
			UpdateAt:    now.Format("2006-01-02 15:04:05"),
		}
	}
	managementDb := implement.NewEggPointPartitionCoefficientManagementDb(db.Db)
	affected, err = managementDb.EggPointPartitionCoefficientManagementBatchInsert(managements)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, nil)
		return
	}
	e.OutSuc(c, affected, nil)
}

// EggPointCoefficientDel
// @Summary      制度中心-蛋蛋分区间系数管理-区间系数(删除)
// @Tags         蛋蛋分区间系数管理
// @Description  区间系数(删除)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req    body	md.EggPointCoefficientDelReq 	true  "需要删除的数据id"
// @Success      200   {int}     "成功删除数据条数"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggPointCoefficient/del [DELETE]
func EggPointCoefficientDel(c *gin.Context) {
	var req md.EggPointCoefficientDelReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}

	managementDb := implement.NewEggPointPartitionCoefficientManagementDb(db.Db)
	affected, err := managementDb.EggPointPartitionCoefficientManagementDel(req.Id)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, nil)
		return
	}
	e.OutSuc(c, affected, nil)
}

// EggPointCoefficientGet
// @Summary      制度中心-蛋蛋分区间系数管理-区间系数(获取)
// @Tags         蛋蛋分区间系数管理
// @Description  区间系数(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Success      200   {object}  md.EggPointCoefficientGetResp
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggPointCoefficient/index [GET]
func EggPointCoefficientGet(c *gin.Context) {
	managementDb := implement.NewEggPointPartitionCoefficientManagementDb(db.Db)
	managements, err := managementDb.EggPointPartitionCoefficientManagementGetAll()
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, nil)
		return
	}
	resp := md.EggPointCoefficientGetResp{}
	if managements == nil {
		e.OutSuc(c, resp, nil)
		return
	}

	list := make([]md.EggPointCoefficientNode, len(*managements))
	for i, management := range *managements {
		list[i] = md.EggPointCoefficientNode{
			Id:          management.Id,
			StartScore:  management.StartScore,
			EndScore:    management.EndScore,
			Coefficient: management.Coefficient,
		}
	}
	resp.List = list
	e.OutSuc(c, resp, nil)
}

// EggPointCoefficientUpdate
// @Summary      制度中心-蛋蛋分区间系数管理-区间系数(更新)
// @Tags         蛋蛋分区间系数管理
// @Description  区间系数(更新)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @param req    body	md.EggPointCoefficientUpdateReq 	true  "需要更新的数据信息"
// @Success      200   {int}     "成功修改的数据数量"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/institutionalManagement/eggPointCoefficient/update [POST]
func EggPointCoefficientUpdate(c *gin.Context) {
	var req md.EggPointCoefficientUpdateReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	var columns []string
	m := model.EggPointPartitionCoefficientManagement{}
	if req.Coefficient != "" {
		m.Coefficient = req.Coefficient
		columns = append(columns, "coefficient")
	}
	if req.StartScore != "" {
		m.StartScore = req.StartScore
		columns = append(columns, "start_score")
	}
	if req.EndScore != "" {
		m.EndScore = req.EndScore
		columns = append(columns, "end_score")
	}
	managementDb := implement.NewEggPointPartitionCoefficientManagementDb(db.Db)
	affected, err := managementDb.EggPointPartitionCoefficientManagementUpdate(req.Id, &m, columns...)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, nil)
		return
	}
	e.OutSuc(c, affected, nil)
}
