package md

import (
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
)

type Paginate struct {
	Limit int   `json:"limit"` // 每页大小
	Page  int   `json:"page"`  // 页数
	Total int64 `json:"total"` // 总数据量
}

type GetWithdrawSettingResp struct {
	Id                      int64                             `json:"id"`
	FrequencySet            md.WithdrawFrequencySettingStruct `json:"frequency_set"`               // 频率设置(0:日 1:周 2:月 3:年)
	WithdrawType            int                               `json:"withdraw_type"`               // 提现方式(1:支付宝 2:微信)
	VipLevelLimit           int                               `json:"vip_level_limit"`             // 提现等级限制
	IsRealName              int                               `json:"is_real_name"`                // 是否实名(0:否 1:是)
	WithdrawNumsLimit       int                               `json:"withdraw_nums_limit"`         // 提现次数限制(对应上方设置周期，0为不限制，失败、驳回不计数 )
	WithdrawAmountLimit     string                            `json:"withdraw_amount_limit"`       // 提现金额限制(0为不限制，大于等于该金额才可以申请提现 )
	WithdrawMultipleLimit   string                            `json:"withdraw_multiple_limit"`     // 提现倍数限制（0为不限制，金额限制倍数才可以申请提现 )
	IsSupportDecimalPoint   int                               `json:"is_support_decimal_point"`    // 是否支持小数点(0:否 1:是)
	IsAuto                  int                               `json:"is_auto"`                     // 是否开启自动提现(0:否 1:是)
	IsAutoAmountLimit       string                            `json:"is_auto_amount_limit"`        // 自动提现金额限制
	WithdrawTimeInterval    md.WithdrawTimeIntervalStruct     `json:"withdraw_time_interval"`      // 提现时段
	WithdrawFeeSet          md.WithdrawFeeSetStruct           `json:"withdraw_fee_set"`            // 提现手续费设置
	FirstWithdrawSet        md.FirstWithdrawSet               `json:"first_withdraw_set"`          // 首次提现设置
	PendingOrdersIsCanApply int                               `json:"pending_orders_is_can_apply"` // 存在待处理记录是否允许再次申请提现(0:禁止,1:允许)
	ConditionIsOpen         int                               `json:"condition_is_open"`           // 提现条件是否开启(0:关闭, 1:开启)
	LevelList               []map[string]interface{}          `json:"level_list"`                  // 会员等级列表
	Tips                    string                            `json:"tips"`                        // 提示
}

type UpdateWithdrawSettingReq struct {
	Id                      int64                             `json:"id,required"`
	FrequencySet            md.WithdrawFrequencySettingStruct `json:"frequency_set"`               // 频率设置(num 个 duration) (duration (0.天 1.小时 2.周 3.月 4.年))
	WithdrawType            int                               `json:"withdraw_type"`               // 提现方式(1:支付宝 2:微信)
	VipLevelLimit           int                               `json:"vip_level_limit"`             // 提现等级限制
	IsRealName              int                               `json:"is_real_name"`                // 是否实名(0:否 1:是)
	WithdrawNumsLimit       int                               `json:"withdraw_nums_limit"`         // 提现次数限制(对应上方设置周期，0为不限制，失败、驳回不计数 )
	WithdrawAmountLimit     string                            `json:"withdraw_amount_limit"`       // 提现金额限制(0为不限制，大于等于该金额才可以申请提现 )
	WithdrawMultipleLimit   string                            `json:"withdraw_multiple_limit"`     // 提现倍数限制（0为不限制，金额限制倍数才可以申请提现 )
	IsSupportDecimalPoint   int                               `json:"is_support_decimal_point"`    // 是否支持小数点(0:否 1:是)
	IsAuto                  int                               `json:"is_auto"`                     // 是否开启自动提现(0:否 1:是)
	IsAutoAmountLimit       string                            `json:"is_auto_amount_limit"`        // 自动提现金额限制
	WithdrawTimeInterval    md.WithdrawTimeIntervalStruct     `json:"withdraw_time_interval"`      // 提现时段(startAt xx:xx endAt xx:xx)
	WithdrawFeeSet          md.WithdrawFeeSetStruct           `json:"withdraw_fee_set"`            // 提现手续费设置
	FirstWithdrawSet        md.FirstWithdrawSet               `json:"first_withdraw_set"`          // 首次提现设置
	PendingOrdersIsCanApply int                               `json:"pending_orders_is_can_apply"` // 存在待处理记录是否允许再次申请提现(0:禁止,1:允许)
	ConditionIsOpen         int                               `json:"condition_is_open"`           // 提现条件是否开启(0:关闭, 1:开启)
	Tips                    string                            `json:"tips"`                        // 提示
}
type GetWithdrawApplyListReq struct {
	Uid             string `json:"uid"`              // 用户 ID
	Nickname        string `json:"nickname"`         // 用户昵称
	Phone           string `json:"phone"`            // 用户手机号
	ParentID        string `json:"parent_id"`        // 推荐人 ID
	IsFirst         string `json:"is_first"`         // 是否首次提现 (0. 否 1.是 传空查全部)
	WithdrawType    string `json:"withdraw_type"`    // 提现方式(1:支付宝 2:微信)
	WithdrawAccount string `json:"withdraw_account"` // 支付宝/微信昵称
	WithdrawName    string `json:"withdraw_name"`    // 支付宝姓名
	PaymentType     string `json:"payment_type"`     // 打款类型 1：手动；2：自动
	State           string `json:"state"`            // 状态 0申请中,1通过,2完成,3失败,4处理中（队列）
	ApplyStartAt    string `json:"apply_start_at"`   // 申请开始时间
	ApplyEndAt      string `json:"apply_end_at"`     // 申请结束时间
	ExamineStartAt  string `json:"examine_start_at"` // 审核开始时间
	ExamineEndAt    string `json:"examine_end_at"`   // 审核结束时间
	AmountBegin     string `json:"amount_begin"`     // 最小金额
	AmountEnd       string `json:"amount_end"`       // 最大金额
	Level           string `json:"level"`            // 会员等级
	Page            int    `json:"page"`             // 页数
	Limit           int    `json:"limit"`            //页面大小
}
type WithdrawApplyAuditReq struct {
	WithdrawApplyId string `json:"withdraw_apply_id" binding:"required"` // 提现申请id
	AuditState      int    `json:"audit_state" binding:"required"`       // 审核状态(1:同意  2:拒绝)
	Reason          int    `json:"reason"`                               // 审核失败（驳回理由）
}

type WithdrawApplyInfo struct {
	model.FinWithdrawApply `xorm:"extends"`
	Nickname               string `xorm:"nickname"`           // 用户名称
	ParentID               int64  `xorm:"parent_uid"`         // 推荐人ID
	AliPayName             string `xorm:"alipay_user_name"`   // 支付宝昵称
	AliPayAccount          string `xorm:"alipay_open_id"`     // 支付宝账号
	WxPayName              string `xorm:"wx_user_name"`       // 微信昵称
	WxPayAccount           string `xorm:"wx_user_id"`         // 微信账号
	InviteCode             string `xorm:"system_invite_code"` // 邀请码
	CustomInviteCode       string `xorm:"custom_invite_code"` // 自定义邀请码
}

type GetWithdrawApplyListResp struct {
	AuditList         []map[string]interface{}   `json:"audit_list"`         // 审核条件
	ReasonList        []map[string]interface{}   `json:"reason_list"`        // 审核失败（驳回理由）
	LevelsList        []map[string]interface{}   `json:"levels_list"`        // 等级列表
	TagsList          []map[string]interface{}   `json:"tags_list"`          // 标签列表
	PendingAmount     string                     `json:"pending_amount"`     // 处理中金额
	PaySucceedAmount  string                     `json:"pay_succeed_amount"` // 打款成功金额
	PayFailedAmount   string                     `json:"pay_failed_amount"`  // 打款失败金额
	UnderReviewAmount string                     `json:"under_review"`       // 审核中金额
	List              []GetWithdrawApplyListNode `json:"list"`
	Paginate          Paginate                   `json:"paginate"` // 分页信息
}

type GetWithdrawApplyListNode struct {
	WithdrawApplyId  string    `json:"withdraw_apply_id"`  // 提现申请id
	UserID           int64     `json:"user_id"`            // 会员 ID
	Nickname         string    `json:"nickname"`           // 用户名称
	ParentID         int64     `xorm:"parent_id"`          // 推荐人ID
	ParentPhone      string    `xorm:"parent_phone"`       // 推荐人手机号
	AliPayName       string    `json:"ali_pay_name"`       // 支付宝昵称
	WechatPayName    string    `json:"wechat_pay_name"`    // 微信昵称
	AliPayAccount    string    `json:"ali_pay_account"`    // 支付宝账号
	WechatPayAccount string    `json:"wechat_pay_account"` // 微信账号
	Tag              []TagNode `json:"tag"`                // 标签
	WithdrawType     int       `json:"withdraw_type"`      // 转账平台(1:支付宝 2:微信)
	InviteCode       string    `json:"invite_code"`        // 邀请码
	Amount           string    `json:"amount"`             // 提现金额
	ActualReceipt    string    `json:"actual_receipt"`     // 实际到账
	SysFee           string    `json:"sys_fee"`            // 手续费
	State            int       `json:"state"`              // 状态 0申请中,1通过,2完成,3失败,4处理中（队列）
	ApplyAt          string    `json:"apply_at"`           // 申请时间
	PayAt            string    `json:"pay_at"`             // 到账时间
	Memo             string    `json:"memo"`               // 备注
}

type TagNode struct {
	TagID   int    `json:"tag_id"`
	TagName string `json:"tag_name"`
}
