package md

import (
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
)

type RegisterReq struct {
	Mobile    string `json:"mobile" binding:"required"`
	Type      string `json:"type" example:"app:APP注册、h5:H5注册 admin 后台"`
	ParentUid string `json:"parent_uid" example:"上级用户"`
	Password  string `json:"password"  example:"密码 如 123"`
	Nickname  string `json:"nickname"`
	Avatar    string `json:"avatar"`
}
type InviteCodeReq struct {
	Id               string `json:"id"`
	CustomInviteCode string `json:"custom_invite_code"`
}
type UserReq struct {
	Id string `json:"id"`
}
type UserManagementGetUserListReq struct {
	ID               int64  `json:"id"` // 会员 ID
	Nickname         string `json:"nickname" example:"会员昵称"`
	RecommendID      int64  `json:"recommend_id"` // 推荐人 ID
	Tag              int    `json:"tag"`          // 标签 id
	Phone            string `json:"phone" example:"会员手机号"`
	InviteCode       string `json:"invite_code" example:"会员邀请码"`
	ParentInviteCode string `json:"parent_invite_code" example:"上级邀请码"`
	ParentPhone      string `json:"parent_phone" example:"上级手机号"`
	Sex              int    `json:"sex"` // 性别(0:未知 1:男 2:女)
	UnionId          string `json:"union_id" example:"微信号"`
	Level            int    `json:"level"`           // 会员等级
	RegisterType     int    `json:"register_type"`   // 注册类型(1:APP注册、2:H5注册)
	State            int    `json:"state"`           // 状态
	Effective        int    `json:"effective"`       // 有效会员
	IsRealName       int    `json:"is_real_name"`    // 是否实名 0.未实名,1.已实名
	Memo             string `json:"memo"`            // 备注
	RegisterBefore   string `json:"register_before"` // 注册时间起点
	RegisterAfter    string `json:"register_after"`  // 注册时间终点
	LoginBefore      string `json:"login_before"`    // 最近登录开始时间
	LoginAfter       string `json:"login_after"`     // 最近登录结束时间
	Page             int    `json:"page,required"`
	Limit            int    `json:"limit,required"`
	IsRecycle        string `json:"is_recycle"`
}

type Paginate struct {
	Limit int   `json:"limit"` // 每页大小
	Page  int   `json:"page"`  // 页数
	Total int64 `json:"total"` // 总数据量
}

type UserManagementGetUserListNode struct {
	ID               int64     `json:"id"`           // 会员 ID
	Tag              []TagNode `json:"tag"`          // 会员标签
	Sex              int       `json:"sex"`          // 性别(0:未知 1:男 2:女)
	Avatar           string    `json:"avatar"`       // 头像
	Nickname         string    `json:"nickname"`     // 昵称
	Phone            string    `json:"phone"`        // 手机号
	IsRealName       int       `json:"is_real_name"` // 是否实名 0.未实名,1.已实名
	LevelName        string    `json:"level_name"`   // 会员等级
	InviteCode       string    `json:"invite_code" example:"会员邀请码"`
	CustomInviteCode string    `json:"custom_invite_code"`
	ParentID         int64     `json:"parent_id"` // 推荐人 ID
	ParentInviteCode string    `json:"parent_invite_code" example:"推荐人邀请码"`
	ParentPhone      string    `json:"parent_phone" example:"推荐人手机号"`
	RegisterTime     string    `json:"register_time"` // 注册时间
	Memo             string    `json:"memo"`          // 备注
	Wechat           string    `json:"wechat"`        // 微信号
	RegisterType     int       `json:"register_type"` // 注册类型
	State            int       `json:"state"`         // 状态
	LastLoginAt      string    `json:"last_login_at"` // 最后登录时间
}

type UserManagementGetUserListResp struct {
	LevelsList []map[string]interface{}        `json:"levels_list"` // 等级列表
	TagsList   []map[string]interface{}        `json:"tags_list"`   // 标签列表
	StateList  []map[string]interface{}        `json:"state_list"`  // 状态列表
	List       []UserManagementGetUserListNode `json:"list"`
	Paginate   Paginate                        `json:"paginate"` // 分页信息
}

type UserInfo struct {
	model.User             `xorm:"extends"`
	ParentPhone            string `xorm:"parent_phone"`
	ParentSystemInviteCode string `xorm:"parent_system_invite_code"`
	ParentCustomInviteCode string `xorm:"parent_custom_invite_code"`
}

type LevelCount struct {
	Level int `xorm:"level"`
	Count int `xorm:"count"`
}

type UserManagementUpdateUserInfoReq struct {
	UID           int64  `json:"uid,required"`
	Avatar        string `json:"avatar"`   // 头像
	Sex           string `json:"sex"`      // 性别(0:未知 1:男 2:女)
	Level         string `json:"level"`    // 会员等级 ID
	Tag           string `json:"tag"`      // 用户标签 ID列表
	Phone         string `json:"phone"`    // 手机号
	Nickname      string `json:"nickname"` // 用户名
	UnionId       string `json:"union_id"` // 微信号
	Memo          string `json:"memo" example:"备注"`
	State         string `json:"state"`         // 账号状态 1正常,2冻结
	LastLoginIp   string `json:"last_login_ip"` // 用户最后登录 IP
	ParentUid     string `json:"parent_uid"`    // 邀请人 ID
	Password      string `json:"password"`      // 登录密码
	WechatAccount string `json:"wechat_account"`
}
type UserManagementUserInfoReq struct {
	UID       string `json:"uid" example:"多个逗号隔开"`
	ParentUid string `json:"parent_uid"  example:"参考下智莺 保留关系时候的选择用户"`
}
type UserManagementUserInfoCancelReq struct {
	UID string `json:"uid" example:"多个逗号隔开"`
}
type TagNode struct {
	TagID   int    `json:"tag_id"`   // 标签 ID
	TagName string `json:"tag_name"` // 标签名称
}

type BasicInfoNode struct {
	Avatar           string    `json:"avatar"`       // 头像
	AvatarUrl        string    `json:"avatar_url"`   // 头像
	Sex              int       `json:"sex"`          // 性别
	Nickname         string    `json:"nickname"`     // 用户名称
	LevelName        string    `json:"level_name"`   // 会员等级名称
	LevelId          int       `json:"levelId"`      // 会员等级 ID
	Tag              []TagNode `json:"tag"`          // 标签名称
	Phone            string    `json:"phone"`        // 手机号
	UnionId          string    `json:"union_id"`     // 微信号
	Password         string    `json:"password"`     // 登录密码
	State            int       `json:"state"`        // 账号状态 1正常,2冻结
	Memo             string    `json:"memo"`         // 备注
	ParentUid        int64     `json:"parent_uid"`   // 推荐人
	ParentName       string    `json:"parent_name"`  // 推荐人名称
	ParentPhone      string    `json:"parent_phone"` // 推荐人手机号
	WechatAccount    string    `json:"wechat_account"`
	SystemInviteCode string    `json:"system_invite_code"`
	CustomInviteCode string    `json:"custom_invite_code"`
	AlipayUserName   string    `json:"alipay_user_name"`
	AlipayOpenId     string    `json:"alipay_open_id"`
	WechatUserName   string    `json:"wechat_user_name"`
	WechatOpenId     string    `json:"wechat_open_id"`
}

type OtherNode struct {
	LastLoginIp string `json:"last_login_ip" ` // 最后登录 IP
	LastLoginAt string `json:"last_login_at" ` // 最近登录时间
	CreateAt    string `json:"create_at" `     // 注册时间
}

type WalletNode struct {
	Amount string `json:"amount"` // 用户余额
}

type VirtualAmountInfoNode struct {
	CoinName string `json:"coin_name"` // 货币类型
	Amount   string `json:"amount"`    // 余额
}

type UserManagementGetOneBasicResp struct {
	TagsList   []map[string]interface{} `json:"tags_list"`   // 标签列表
	LevelsList []map[string]interface{} `json:"levels_list"` // 等级列表
	BasicInfo  BasicInfoNode            `json:"basic_info"`  // 基本信息
	OtherInfo  OtherNode                `json:"other_info"`  // 其他信息
}

type FansNode struct {
	ID          int64  `json:"id"` // 用户ID
	Nickname    string `json:"nickname" example:"用户昵称"`
	Phone       string `json:"phone" example:"手机号"`
	LevelTotal  int    `json:"level_total"`  // 所属代数
	Amount      string `json:"amount"`       // 可提现余额
	TotalIncome string `json:"total_income"` // 累计收益
	RegisterAt  string `json:"register_at"`  // 注册时间
}

type UserManagementGetFansResp struct {
	List     []FansNode `json:"list"`     // 用户数据
	Paginate Paginate   `json:"paginate"` // 分页信息
}

type BalanceDetailNode struct {
	ID           int64  `json:"id"`            // 记录ID
	Amount       string `json:"amount"`        // 变更数量
	BeforeAmount string `json:"before_amount"` // 变更前余额
	AfterAmount  string `json:"after_amount"`  // 变更后余额
	Kind         string `json:"kind"`          //类型 1:管理员操作增加余额 2:管理员操作扣除余额 3:蛋蛋能量兑换余额 4:余额兑换蛋蛋能量
	CreateAt     string `json:"create_at"`     // 创建时间
}

type UserManagementGetBalanceDetailResp struct {
	List     []BalanceDetailNode `json:"list"`
	Paginate Paginate            `json:"paginate"`
}

type VirtualCoinDetailNode struct {
	Uid         int64  `json:"uid"`          // 用户 ID
	Amount      string `json:"amount"`       // 变更数量
	AfterAmount string `json:"after_amount"` // 变更后积分余额
	Direction   int    `json:"direction"`    // 类型 1.收入 2.支出
	CreateAt    string `json:"create_at"`    // 创建时间
}

type UserManagementGetVirtualCoinDetailResp struct {
	CoinList []map[string]interface{} `json:"coin_list"`
	List     []VirtualCoinDetailNode  `json:"list"`
	Paginate Paginate                 `json:"paginate"`
}

type UserManagementAddTagReq struct {
	Uid   int64  `json:"uid"`    // 用户 ID
	TagId string `json:"tag_id"` // 标签 ID
	Memo  string `json:"memo"`   // 标签备注
}

type UserManagementDelTagReq struct {
	Uid   int64  `json:"uid"`    // 用户 ID
	TagId string `json:"tag_id"` // 标签 ID
}
