package svc

import (
	md2 "applet/app/md/institutional_management/egg_energy"
	"applet/app/svc/egg_energy"
	"applet/app/utils/cache"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_es.git/es"
	"context"
	"fmt"
	"strings"
	"time"
	"xorm.io/xorm"
)

const AutoCachingEggPointStatisticsLockKey = "egg_energy_auto_automatic_scoring_lock_key"

func AutoCachingEggPointStatistics(engine *xorm.Engine) {
	fmt.Println("svc_egg_energy_auto_caching_egg_point_statistics...")
	defer func() {
		if err := recover(); err != nil {
			fmt.Println(err)
			return
		}
	}()
	fmt.Println("----------------------------AutoCachingEggPointStatistics_begin-------------------------------")

	getString, _ := cache.GetString(AutoCachingEggPointStatisticsLockKey)
	if getString != "" {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "上一次蛋蛋分统计未执行完")
		return
	}
	cache.SetEx(AutoCachingEggPointStatisticsLockKey, "running", 10*60) // 10 min
	defer cache.Del(AutoCachingEggPointStatisticsLockKey)

	// 1. 查询所有蛋蛋分索引
	aliasName := md.EggEnergyUserEggScoreEsAlias
	aliases, err := es.EsClient.Aliases().
		Index(aliasName). // 指定别名
		Do(context.Background())
	if err != nil {
		fmt.Println("AutoCachingEggPointStatistics_GetAllIndexFailed_ERR:::::", err.Error())
		return
	}
	now := time.Now()
	year, week := now.ISOWeek()
	currentWeekIndex := fmt.Sprintf(md.EggEnergyUserEggScoreEsAlias+"_%d%d", year, week)
	// 2. 按周处理蛋蛋分记录
	for esIndexName, _ := range aliases.Indices {
		// 2.1 查询是否有此周且不为当周记录
		egg_energy.GetYearsAndWeekStr(esIndexName)
		if esIndexName != currentWeekIndex {
			exists := cache.Exists(md2.AutoCachingEggPointRedisKey)
			if exists {
				continue
			}
		}

		// 2.2 更新记录
		var tempWeek md2.StatisticsEggPointResp
		//统计用户"ecpm"范围
		tempWeek.StatisticsUserEggEcpmRange, err = egg_energy.StatisticsUserEggEcpmRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"拉新人数"范围
		tempWeek.StatisticsUserEggInviteUserNumsRange, err = egg_energy.StatisticsUserEggInviteUserNumsRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"团队活跃次数"范围
		tempWeek.StatisticsUserEggTeamActivityNumsRange, err = egg_energy.StatisticsUserEggTeamActivityNumsRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"签到次数"范围
		tempWeek.StatisticsUserEggSignInNumsRange, err = egg_energy.StatisticsUserEggSignInNumsRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"发红包次数"范围
		tempWeek.StatisticsUserEggSendRedPackageNumsRange, err = egg_energy.StatisticsUserEggSendRedPackageNumsRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"蛋蛋能量兑换余额数量"范围
		tempWeek.StatisticsUserEggEggEnergyExchangeAccountBalanceRange, err = egg_energy.StatisticsUserEggEggEnergyExchangeAccountBalanceRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"蛋蛋能量兑换余额数量"范围
		tempWeek.StatisticsUserEggAccountBalanceExchangeEggEnergyNumsRange, err = egg_energy.StatisticsUserEggAccountBalanceExchangeEggEnergyNumsRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"发朋友圈次数"范围
		tempWeek.StatisticsUserEggSendCircleOfFriendNumsRange, err = egg_energy.StatisticsUserEggSendCircleOfFriendNumsRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"论坛评论次数"范围
		tempWeek.StatisticsUserEggForumCommentsNumsRange, err = egg_energy.StatisticsUserEggForumCommentsNumsRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"学院学习次数"范围
		tempWeek.StatisticsUserEggCollegeLearningNumsRange, err = egg_energy.StatisticsUserEggCollegeLearningNumsRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"违规次数"范围
		tempWeek.StatisticsUserEggViolateNumsRange, err = egg_energy.StatisticsUserEggViolateNumsRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"浏览界面次数"范围
		tempWeek.StatisticsUserEggBrowseInterfaceNumsRange, err = egg_energy.StatisticsUserEggBrowseInterfaceNumsRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}
		//统计用户"个人活跃积分增量值"范围
		tempWeek.StatisticsUserEggPersonAddActivityValueRange, err = egg_energy.StatisticsUserEggPersonAddActivityValueRange(esIndexName)
		if err != nil {
			fmt.Println("AutoCachingEggPointStatistics_StatisticsFailed_ERR:::::", err.Error())
		}

		parts := strings.Split(esIndexName, "_")
		yearAndWeekStr := parts[len(parts)-1]
		redisKey := fmt.Sprintf(md2.AutoCachingEggPointRedisKey, yearAndWeekStr)
		// 更新缓存
		cache.Del(redisKey)
		cache.SetJson(redisKey, tempWeek, -1)
	}
	fmt.Println("----------------------------AutoCachingEggPointStatistics_end-------------------------------")
}
