package svc

import (
	"applet/app/utils"
	"applet/app/utils/cache"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/utils/es"
	es2 "code.fnuoos.com/go_rely_warehouse/zyos_go_es.git/es"
	"fmt"
	"time"
	"xorm.io/xorm"
)

const deleteLimit = 1000
const EggEnergyAutoUpdateUserAccessLockKey = "egg_energy_auto_update_user_access_lock_key"

// EggEnergyAutoUpdateUserAccess 自动更新用户访问次数记录
func EggEnergyAutoUpdateUserAccess(engine *xorm.Engine) {
	fmt.Println("egg_energy_auto_update_user_access...")
	defer func() {
		if err := recover(); err != nil {
			fmt.Println(err)
			return
		}
	}()
	fmt.Println("----------------------------EggEnergyAutoUpdateUserAccess_begin-------------------------------")

	now := time.Now()
	if !(now.Hour() >= 2 && now.Hour() < 6) {
		//TODO::只在凌晨两点 ~ 凌晨六点运行
		return
	}
	getString, _ := cache.GetString(EggEnergyAutoUpdateUserAccessLockKey)
	if getString != "" {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "上一次蛋蛋分统计未执行完")
		return
	}
	cache.SetEx(EggEnergyAutoUpdateUserAccessLockKey, "running", 4*60*60) // 4 h
	defer cache.Del(EggEnergyAutoUpdateUserAccessLockKey)

	// 获取当周信息
	year, week := time.Now().ISOWeek()
	yearStr := utils.IntToStr(year)
	weekStr := utils.IntToStr(week)
	index := es.GetAppointIndexFromAlias(yearStr, weekStr)
	page := 1
	limit := 100
	totalDb := implement.NewEggAccessTotalDb(engine)
	for {
		list, err := totalDb.EggAccessTotalGetByPage(year, week, page, limit)
		if err != nil {
			fmt.Println("EggEnergyAutoUpdateUserAccess_GetAccessTotalFailed:::", err)
			return
		}
		ids := make([]string, 0, len(list))
		updateInfoMap := make([]interface{}, 0, len(list))
		for _, node := range list {
			uid := node.Uid
			id := fmt.Sprintf("%d%d_%d", year, week, uid)
			ids = append(ids, id)
			updateInfoMap = append(updateInfoMap, map[string]interface{}{
				"browse_interface_nums": node.Total,
			})
		}
		updateBulkDocRet, err := es2.UpdateBulkDoc(index, ids, updateInfoMap)
		if err != nil {
			fmt.Println("EggEnergyAutoUpdateUserAccess_UpdateEsFailed:::", err)
			return
		}
		fmt.Printf("updateBulkDocRet ==> %+v \n\n", updateBulkDocRet)
		if len(list) < limit {
			break
		} else {
			page++
		}
	}
	// 删除前一天记录
	for {
		toDeleteDateStr := now.AddDate(0, 0, -1).Format("2006-01-02")
		recordsDb := implement.NewEggAccessRecordsDb(engine)
		affected, err := recordsDb.EggAccessRecordsDeleteByDateByPage(toDeleteDateStr, deleteLimit)
		if err != nil {
			fmt.Println("EggEnergyAutoUpdateUserAccess_DeleteAccessRecordFailed:::", err)
			return
		}
		if affected < deleteLimit {
			break
		}
		_, err = engine.Exec("OPTIMIZE table egg_access_records")
		if err != nil {
			fmt.Println("EggEnergyAutoUpdateUserAccess_OPTIMIZEFailed:::", err)
			return
		}
	}

	fmt.Println("----------------------------EggEnergyAutoUpdateUserAccess_end-------------------------------")

}
