package svc

import (
	"applet/app/utils"
	"applet/app/utils/cache"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/md"
	md2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/utils/es"
	es2 "code.fnuoos.com/go_rely_warehouse/zyos_go_es.git/es"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"context"
	"encoding/json"
	"fmt"
	"github.com/olivere/elastic/v7"
	"time"
	"xorm.io/xorm"
)

const EggEnergyAutomaticScoringLockKey = "egg_energy_auto_automatic_scoring_lock_key"

func EggEnergyAutomaticScoring(engine *xorm.Engine) {
	fmt.Println("svc_egg_energy_auto_score...")
	defer func() {
		if err := recover(); err != nil {
			fmt.Println(err)
			return
		}
	}()

	fmt.Println("----------------------------EggEnergyAutomaticScoring_begin-------------------------------")
	// 增加“悲观锁”防止串行
	getString, _ := cache.GetString(EggEnergyAutomaticScoringLockKey)
	if getString != "" {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "上一次蛋蛋分自动打分未执行完")
		return
	}
	cache.SetEx(EggEnergyAutomaticScoringLockKey, "running", 3600*6) // 6小时
	defer cache.Del(EggEnergyAutomaticScoringLockKey)

	now := time.Now()
	weekday := now.Weekday()
	if !(weekday != time.Wednesday && now.Hour() >= 0 && now.Hour() < 4) {
		//TODO::只在周三凌晨零点 ~ 凌晨六点运行 (预计六点可以全部更新完，减少查询次数)
		return
	}

	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		fmt.Println("EggEnergyAutomaticScoringGetRabbitChannel_ERR:::::", err.Error())
		return
	}
	defer ch.Release()

	lastWeek := now.AddDate(0, 0, -7)
	year, week := lastWeek.ISOWeek()
	yearString := utils.IntToStr(year)
	weekString := utils.IntToStr(week)
	LastWeekIndex := es.GetAppointIndexFromAlias(yearString, weekString)

	// 1. 获取上周未被打分的文档
	page := 1
	limit := 100
	var limit64 int64
	limit64 = int64(limit)
	for {
		boolQuery := elastic.NewBoolQuery()
		boolQuery.Filter(elastic.NewTermQuery("score_value", "0"))
		searchResult, err := es2.EsClient.Search().
			Index(LastWeekIndex).
			Query(boolQuery).
			From((page-1)*limit).Size(limit).
			Sort("created_at", true).
			Pretty(true).
			Do(context.Background())
		if err != nil {
			fmt.Println("EggEnergyAutomaticScoring_ERR:::::", err.Error())
			return
		}
		// 检查是否有结果
		if searchResult.Hits.TotalHits.Value != 0 {
			// 解析结果
			for _, hit := range searchResult.Hits.Hits {
				var doc md.EggEnergyUserEggScoreEs
				err = json.Unmarshal(hit.Source, &doc)
				if err != nil {
					return
				}
				m := md2.EggStructForAutoScoreData{
					DocId:                               hit.Id,
					Ecpm:                                doc.Ecpm,
					InviteUserNums:                      doc.InviteUserNums,
					TeamActivityNums:                    doc.TeamActivityNums,
					SignInNums:                          doc.SignInNums,
					ImActivityNums:                      doc.ImActivityNums,
					SendRedPackageNums:                  doc.SendRedPackageNums,
					EggEnergyExchangeAccountBalance:     doc.EggEnergyExchangeAccountBalance,
					AccountBalanceExchangeEggEnergyNums: doc.AccountBalanceExchangeEggEnergyNums,
					SendCircleOfFriendNums:              doc.SendCircleOfFriendNums,
					ForumCommentsNums:                   doc.ForumCommentsNums,
					CollegeLearningNums:                 doc.CollegeLearningNums,
					ViolateNums:                         doc.ViolateNums,
					BrowseInterfaceNums:                 doc.BrowseInterfaceNums,
					PersonAddActivityValue:              doc.PersonAddActivityValue,
				}
				// 推入队列异步处理
				ch.Publish(md2.EggAppExchange, m, md2.EggRoutKeyForAutoScoreData)
			}
		}
		if searchResult.Hits.TotalHits.Value < limit64 {
			break
		} else {
			page++
		}
	}
	nextWeekTime := now.AddDate(0, 0, 7)
	nextYear, nextWeek := nextWeekTime.ISOWeek()
	nextYearString := utils.IntToStr(nextYear)
	nextWeekString := utils.IntToStr(nextWeek)
	nextWeekIndex := es.GetAppointIndexFromAlias(nextYearString, nextWeekString)
	err = es2.CreateIndexIfNotExists(nextWeekIndex, md.EggEnergyUserEggScoreEsMapping)
	if err != nil {
		fmt.Println("EggEnergyAutomaticScoringCreateIndex_ERR_FailedToCreateNextWeekIndex:::::", err.Error())
	}
	fmt.Println("------------------EggEnergyAutomaticScoring_end:finish automatic scoring------------------")
	return
}
