package svc

import (
	"applet/app/utils"
	"applet/app/utils/cache"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	md2 "code.fnuoos.com/EggPlanet/egg_system_rules.git/rule/egg_energy/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"fmt"
	"github.com/shopspring/decimal"
	"time"
	"xorm.io/xorm"
)

const EggEnergyDealPlatformRevenueDataKey = "egg_energy_deal_platform_revenue_data"

// EggEnergyDealPlatformRevenueData 蛋蛋能量-处理平台收益
func EggEnergyDealPlatformRevenueData(eg *xorm.Engine) {
	fmt.Println("egg_energy_deal_platform_revenue_data...")
	defer func() {
		if err := recover(); err != nil {
			fmt.Println("EggEnergyDealPlatformRevenueDataGetRabbitChannel_ERR:::::", err)
			return
		}
	}()

	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		fmt.Println("EggEnergyDealPlatformRevenueDataGetRabbitChannel_ERR:::::", err.Error())
		return
	}
	defer ch.Release()

	// 悲观锁防止串行
	getString, _ := cache.GetString(EggEnergyDealPlatformRevenueDataKey)
	if getString != "" {
		fmt.Println("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", "Platform Revenue Data 上一次处理未执行完")
		return
	}
	cache.SetEx(EggEnergyDealPlatformRevenueDataKey, "running", 60*30) //30分钟
	defer cache.Del(EggEnergyDealPlatformRevenueDataKey)

	// 查询所有未被执行完的数据
	platformRevenueDataDb := implement.NewPlatformRevenueDataDb(eg)
	platformRevenueDataList, err := platformRevenueDataDb.PlatformRevenueDataFindNotFinish()
	if err != nil {
		fmt.Println("EggEnergyDealPlatformRevenueData_ERR:::::", err.Error())
		return
	}

	now := time.Now()
	recordsDb := implement.NewPlatformRevenueDataRecordsDb(eg)
	for _, data := range platformRevenueDataList {
		// 1、判断是否是第一次执行
		lastRecord, err1 := recordsDb.PlatformRevenueDataRecordsGetLast(data.Id)
		if err1 != nil {
			fmt.Println("EggEnergyDealPlatformRevenueData_ERR:::::", err1.Error())
			continue
		}
		if lastRecord == nil {
			// 2.1 首次执行 判断当前是否可以执行
			if utils.TimeParseStd(data.CreateAt).Add(time.Duration(data.Frequency) * time.Minute).After(now) {
				continue
			}
		} else {
			// 2.2 已经被执行过
			if utils.TimeParseStd(lastRecord.CreateAt).Add(time.Duration(data.Frequency) * time.Minute).After(now) {
				continue
			}
		}

		// 3、计算总执行次数、每次更新金额 (每次更新金额 = 金额 / 总执行次数   总执行次数 = 时间 / 频次)
		frequency := decimal.NewFromInt(int64(data.Frequency))
		times := decimal.NewFromInt(int64(data.Hours * 60)).Div(frequency).RoundFloor(8)
		totalAmount, err2 := decimal.NewFromString(data.TotalAmount)
		if err2 != nil {
			fmt.Println("EggEnergyDealPlatformRevenueData_ERR:::::", err2.Error())
			continue
		}
		amount := totalAmount.Div(times)

		// 4、推入rabbitmq 异步处理
		ch.Publish(md2.EggEnergyExchange, utils.SerializeStr(md2.EggEnergyStructForPlatformRevenueData{
			ID:     data.Id,
			Amount: amount.String(),
		}), md2.EggEnergyRoutKeyForPlatformRevenueData)
	}
}
